#!/usr/bin/env python
import sys
import matplotlib
matplotlib.use('Agg')
import matplotlib.pyplot as plt
import numpy as np
import csv
import string
import math
import itertools
from csv import DictReader
from copy import copy
from collections import defaultdict

LINE_STYLE = ["s-","^--","o:", "^-","^:", "o--","o:", ".:", "p:","o-","s:","^:"]
LINE_COLOR = ["k", "#88e2df","#2e5aa8", "b","#d95f02", "#d95f02", "#1b9e77",  "#1b9e77","#7570b3", "#333333","r","b","m"]

chosen_lines = ['NOLOCK','PF-T-Overhead','PF-L-Overhead']

WIDTH=5
HEIGHT=3

def chooseLabel(colName):
    if colName == "no-blocking":
        return "no synchronization"
    elif colName == "PF-T-Overhead":
        return "LP+PF-T"
    elif colName == "PF-L-Overhead":
        return "LP+PF-L"
    elif colName == "TL2":
        return "TL2"
    elif colName == "TL2-collapse":
        return "TL2-Collapse"
    else:
        return colName

def main():
        
    x = []
    y = [[] for _ in range(0, len(chosen_lines))]

    for file in sys.argv[1:]:
        # Get the experiment names
        f = open(file, 'r')
        for row in f:
            if string.join(row).strip().startswith('#') and "TASK" in row:
                title_row = row
        title_row = title_row[1:]
        split_title = title_row.split()
        split_title = split_title[1:]
        req_num = file.split("nreq=")[1].split("__write")[0] 
        f.close()


        f = open(file, 'r')

        plots = csv.reader(f, delimiter=',')
        for row in plots:
            if not string.join(row).strip().startswith('#'):
                #print(row)
                x.append(float(req_num))
                for i in range(0,len(row)-1):
                    y[i].append(float(row[i+1]))
                    #print(y[i])
    if len(x)>1:
      #  lists = sorted(itertools.izip(*[x,y]))
    #    print x
    #    print y
        new_y = [[] for _ in range(0, len(chosen_lines))]
        for i in range(len(new_y)):
            new_x, temp_y = zip(*sorted(zip(x,y[i])))
            new_y[i] = temp_y
    #    print new_x
    #    print new_y
        fig, ax = plt.subplots()
        fig.set_size_inches(WIDTH, HEIGHT)
        for i in range(0, len(split_title)):
            if split_title[i] in chosen_lines:
                #print(x, y[i])
                ax.plot(new_x,new_y[i], LINE_STYLE[i], color=LINE_COLOR[i], label=chooseLabel(split_title[i]))

        plt.legend(loc="upper right")
        plt.xticks([20,200,400,600,800,1000,1200,1400,1600,1800,2000])
    #    plt.xticks(np.arange(min(new_x),max(new_x)+1, step=int((max(new_x)-min(new_x))/10)))
        plt.yticks(np.arange(0,1.1,step=0.1))

        ax.grid(linestyle="dotted")
        ax.spines["top"].set_visible(False)
        ax.spines["right"].set_visible(False)
    
        plt.ylabel("Schedulability Ratio")
        plt.xlabel("Number of Requests")

    # Try to make sure the x-axis label isn't cut off
        plt.tight_layout()

        plt.savefig(file[:-4]+".pdf", bbox_inches = 'tight', pad_inches = 0)
        plt.savefig(file[:-4]+".png", bbox_inches = 'tight', pad_inches = 0)
        plt.close()

if __name__ == '__main__':
    main()
